# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 TU Wien.
#
# Invenio-App-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Record migration script from InvenioRDM 7.0 to 8.0.

Disclaimer: This script is intended to be executed *only once*, namely when
upgrading from InvenioRDM 7.0 to 8.0!
If this script is executed at any other time, probably the best case scenario
is that nothing happens!
"""

from pathlib import Path

import invenio_rdm_records.fixtures as fixtures
from click import echo, secho
from invenio_access.permissions import system_identity
from invenio_db import db
from invenio_rdm_records.fixtures import PrioritizedVocabulariesFixtures, \
    VocabulariesFixture
from invenio_vocabularies.proxies import current_service as vocabulary_svc


def execute_upgrade():
    """Execute the upgrade from InvenioRDM 7.0 to 8.0.

    Please read the disclaimer on this module before thinking about executing
    this function!
    """

    def update_license(license_dict):
        """Update the stored vocabulary with the new icon."""
        try:
            echo(f"Updating license: {license_dict['id']}... ", nl=False)
            license_ = vocabulary_svc.read(
                system_identity, ("licenses", license_dict["id"])
            )._obj

            # NOTE: we don't use the service update method here because we
            # want to evade validation errors, and the migration guide tells
            # the users to completely rebuild the search indices anyway
            # and we pop the '$schema' because it might be outdated and is
            # a constant field anyway
            license_["icon"] = license_dict["icon"]
            license_.pop("$schema", None)
            license_.commit()
            secho("OK", fg="green")

        except Exception as e:
            secho("Error", fg="red")
            return f"Error updating license '{license_dict['id']}': {e}"

        return None

    def update_licenses_from_fixture(fixture):
        """Use the given fixture to update the license vocabularies."""
        errors = []
        did_load_licenses = False

        for id_, entry in fixture.read():
            if id_ == "licenses":
                for license_dict in entry.iterate(ignore=[]):
                    did_load_licenses = True
                    error = update_license(license_dict)
                    if error:
                        errors.append(error)

        return did_load_licenses, errors

    # let the prioritized vocabularies fixture take care of the
    # path building and entrypoint definition, etc.
    dir_ = Path(fixtures.__file__).parent
    pf = PrioritizedVocabulariesFixtures(
        system_identity,
        app_data_folder=Path("./app_data"),
        pkg_data_folder=(dir_ / "data"),
        filename="vocabularies.yaml",
    )

    errors = []
    licenses_loaded = False

    # we're checking the same places as the prioritized vocabularies fixtures,
    # and in the same order
    # 1: check the app_data directory for the invenio instance
    app_data_yaml_file = pf._app_data_folder / pf._filename
    if app_data_yaml_file.exists():
        app_data_fixture = VocabulariesFixture(
            system_identity, app_data_yaml_file
        )
        licenses_loaded, errs = update_licenses_from_fixture(app_data_fixture)
        errors.extend(errs)

    # 2: check the entry points
    extensions = [ep.load() for ep in pf._entry_points()]
    for module in extensions:
        directory = Path(module.__file__).parent
        filepath = directory / pf._filename
        vocab_list = pf.peek_vocabularies(filepath)

        # check if the entry points define something for licenses
        if not licenses_loaded and "licenses" in vocab_list:
            extension_fixture = VocabulariesFixture(system_identity, filepath)
            licenses_loaded, errs = update_licenses_from_fixture(
                extension_fixture
            )
            errors.extend(errs)

    # 3: check the default vocabularies from rdm-records
    pkg_data_yaml_file = pf._pkg_data_folder / pf._filename
    if not licenses_loaded and pkg_data_yaml_file.exists():
        pkg_data_fixture = VocabulariesFixture(
            system_identity, pkg_data_yaml_file
        )
        licenses_loaded, errs = update_licenses_from_fixture(pkg_data_fixture)
        errors.extend(errs)

    success = not errors

    # give feedback on the operation
    if not licenses_loaded:
        secho(
            "Warning: No licenses were upgraded, which is unexpected.",
            fg="yellow",
        )

    if success:
        db.session.commit()
        secho(
            "Data migration completed, please rebuild the search indices now.",
            fg="green",
        )

    else:
        db.session.rollback()
        secho(
            "Upgrade aborted due to the following errors:",
            fg="red",
            err=True,
        )

        for error in errors:
            secho(error, fg="red", err=True)

        msg = (
            "The changes have been rolled back. "
            "Please fix the above listed errors and try the upgrade again",
        )
        secho(msg, fg="yellow", err=True)


# if the script is executed on its own, perform the upgrade
if __name__ == "__main__":
    execute_upgrade()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Record migration script from InvenioRDM 3.0 to 4.0.

Disclaimer: This script is intended to be executed *only once*, namely when
upgrading from InvenioRDM 3.0 to 4.0!
If this script is executed at any other time, probably the best case scenario
is that nothing happens!
"""

from invenio_db import db
from invenio_rdm_records.records.api import RDMDraft, RDMParent, RDMRecord


def execute_upgrade():
    """Execute the upgrade from InvenioRDM 3.0 to 4.0.

    Please read the disclaimer on this module before thinking about executing
    this function!
    """

    def remove_duplicate_languages(record):
        """Remove duplicate languages."""
        if "languages" in record["metadata"]:
            serialized_languages = map(
                tuple,
                map(sorted, map(dict.items, record["metadata"]["languages"])),
            )
            unique_languages = set(serialized_languages)
            languages_list = list(map(dict, unique_languages))
            record["metadata"]["languages"] = languages_list

    def update_vocabularies(record):
        """Updates languages and resource_type to become vocabularies."""

        def get_res_type_vocabulary(data):
            """Returns the id value of the resource type vocabulary."""
            if "subtype" in data["resource_type"]:
                return data["resource_type"]["subtype"]
            elif "type" in data["resource_type"]:
                return data["resource_type"]["type"]

        def get_language_vocabulary(data):
            """Returns the language as vocabulary."""
            return dict(id=data)

        def migrate_language(field):
            """Migrates language field."""
            for idx, val in enumerate(record["metadata"].get(field, [])):
                if "lang" in val:
                    language_vocab = get_language_vocabulary(val["lang"])
                    record["metadata"][field][idx]["lang"] = language_vocab

        # Migrate resource_type
        if "resource_type" in record["metadata"]:
            res_type_vocab = get_res_type_vocabulary(record["metadata"])
            record["metadata"]["resource_type"] = dict(id=res_type_vocab)

        # Migrate resource_type of related_identifiers
        for idx, val in enumerate(
            record["metadata"].get("related_identifiers", [])
        ):
            if "resource_type" in val:
                res_type_vocab = get_res_type_vocabulary(val)
                record["metadata"]["related_identifiers"][idx][
                    "resource_type"
                ] = dict(id=res_type_vocab)

        # Migrate languages from additional_descriptions
        migrate_language("additional_descriptions")
        # Migrate languages from additional_titles
        migrate_language("additional_titles")

        record.commit()

    for record_metadata in RDMRecord.model_cls.query.all():
        record = RDMRecord(record_metadata.data, model=record_metadata)

        remove_duplicate_languages(record)

        # Updating to new $schema when eventually saved
        record.pop("$schema", None)

        # Adding empty pids
        if record.pids is None:
            record.pids = {}

        record.commit()

    for draft_metadata in RDMDraft.model_cls.query.all():
        # Skipping deleted drafts because can't be committed
        if draft_metadata.is_deleted:
            continue

        draft = RDMDraft(draft_metadata.data, model=draft_metadata)

        remove_duplicate_languages(draft)

        # Updating to new $schema when eventually saved
        draft.pop("$schema", None)

        # Adding empty pids
        if draft.pids is None:
            draft.pids = {}

        draft.commit()

    db.session.commit()

    # Need to loop again to update the resource type once the scheme is updated
    for record_metadata in RDMRecord.model_cls.query.all():
        record = RDMRecord(record_metadata.data, model=record_metadata)

        update_vocabularies(record)

    for draft_metadata in RDMDraft.model_cls.query.all():
        # Skipping deleted drafts because can't be committed
        if draft_metadata.is_deleted:
            continue

        draft = RDMDraft(draft_metadata.data, model=draft_metadata)

        update_vocabularies(draft)

    for parent_metadata in RDMParent.model_cls.query.all():
        parent = RDMParent(parent_metadata.data, model=parent_metadata)

        # Updating to new $schema when eventually saved
        parent.pop("$schema", None)

        parent.commit()

    db.session.commit()


if __name__ == "__main__":
    execute_upgrade()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Request views module."""

from flask import render_template
from flask_babelex import lazy_gettext as _
from invenio_communities.views.decorators import pass_community, \
    pass_community_logo


@pass_community
@pass_community_logo
def communities_detail(community=None, logo=None, pid_value=None):
    """Community detail page."""
    permissions = community.has_permissions_to(
        ['update', 'read', 'search_requests', 'search_invites']
    )
    endpoint = '/api/communities/{pid_value}/records'
    return render_template(
        "invenio_communities/details/index.html",
        community=community.to_dict(),  # TODO: use serializer
        logo=logo.to_dict() if logo else None,
        # TODO: inject this from a vocabulary in the community
        types={
            "organization": _("Organization"),
            "event": _("Event"),
            "topic": _("Topic"),
            "project": _("Project")
        },
        # Pass permissions so we can disable partially UI components
        # e.g Settings tab
        permissions=permissions,
        active_menu_tab="search",
        endpoint=endpoint.format(pid_value=community.to_dict()["uuid"])
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CERN.
# Copyright (C) 2019 Northwestern University.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Simple test of version import."""


def test_version():
    """Test version import."""
    from invenio_app_rdm import __version__
    assert __version__
